//+------------------------------------------------------------------+
//|                                                      Balance.mqh |
//|                                            Copyright 2010, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <Balance.mqh>                                           |
//+------------------------------------------------------------------+
//#include <Decode MarketWatch.mqh>
#include <Arrays\ArrayLong.mqh>
//+------------------------------------------------------------------+
//| structure of storing balance information                         |
//+------------------------------------------------------------------+
struct SADtD
  {
public:
                     SADtD(){};
                    ~SADtD(){};
   datetime          time;
   double            balance;
   double            open;
   double            bar1;
   double            bar2;
   double            close;
  };
//+------------------------------------------------------------------+
//| CBalance                                                         |
//+------------------------------------------------------------------+
class CBalance
  {
   int               step;    // step of change of reserving as 2^step // set in the constructor
   int               reserved;// value of memory reserve
public:
   int               bsize;   // size of the 'array' array of structures
   SADtD             array[];
                     CBalance(void)
     {
      step=5;// set the step of change of reserving as 2^step
      reserved=0;
      bsize=0;
      Resize(0);
     };
                    ~CBalance(void){};
   int               Resize(int new_size);
   int               WriteFile(string folder_);
   int               ReadFile(string folder_);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CBalance::Resize(int new_size)
  {
   bsize=new_size;
   if((bsize>>step<<step)<reserved)
     {
      ArrayResize(array,bsize);
     }
   else
     {
      reserved=((bsize>>step)+1)<<step;
      int new_reserv=reserved-bsize;// amount of memory to be additionally reserved      
      ArrayResize(array,bsize,new_reserv);
     }
   return(bsize);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CBalance::WriteFile(string folder_)
  {
   int han=FileOpen(folder_+".dat",FILE_WRITE|FILE_BIN);
   if(han!=INVALID_HANDLE)
     {
      bsize=(int)FileWriteArray(han,array);
      FileClose(han);
     }
   else Print(__FUNCTION__+" File open failed "+folder_+".dat, error",GetLastError());
   ResetLastError();
   return(bsize);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CBalance::ReadFile(string folder_)
  {
   int han=FileOpen(folder_+".dat",FILE_READ|FILE_BIN);
   if(han!=INVALID_HANDLE)
     {
      bsize=(int)FileReadArray(han,array);
      FileClose(han);
     }
   else Print(__FUNCTION__+" File open failed "+folder_+".dat, error",GetLastError());
   ResetLastError();
   return(bsize);
  }
//+------------------------------------------------------------------+
//| structure of history data                                        |
//+------------------------------------------------------------------+
struct SHistData
  {
public:
                     SHistData(){};
                    ~SHistData(){};
   datetime          time;
   double            price;
   double            balance;
   double            entry;
   double            volume;
  };
//+------------------------------------------------------------------+
//| CSymBalance                                                      |
//+------------------------------------------------------------------+
class CSymBalance : public CBalance
  {

   string            folder;
   double            iPoint;
   string            name_report;
   string            symbol;
   datetime TimeBars(int bar){datetime temp[];CopyTime(symbol,PERIOD_M1,bar,1,temp);return(temp[0]);};

public:
   MqlRates          rates[];// declare the array of structures
   int               count_rates;
   SHistData         hdata[];
   int               hsize;
   datetime          beg;
   datetime          end;

                     CSymBalance(void){hsize=0;};
                    ~CSymBalance(void){};
   //--- reading trade history of a symbol
   bool              ReadHistory(string folder_);
   //--- getting M1 data for the specified symbol
   bool              InitData(string name_report_,string symbol_);
   //--- calculation of balance and equity information on the basis of M1 history and trade information
   int               Calculate_Balance();
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSymBalance::ReadHistory(string folder_)
  {
// time,price,balance,entry,volume
//   0    4      6      9     10   
   bool res=false;   string temp,text="";
   int han=FileOpen(folder_+".csv",FILE_READ|FILE_SHARE_READ|FILE_ANSI,"\n");
   if(han!=INVALID_HANDLE)
     {
      while(!IsStopped())
        {
         text=FileReadString(han);

         if(text!="")
           {
            int len=StringLen(text);
            int glob=0;
            int c=0,prevs=0;//";"-59

            hsize=ArrayResize(hdata,hsize+1);

            for(int i=0;i<len;i++)
              {
               c++;
               if(StringGetCharacter(text,i)==59)
                 {
                  i++; if(glob==0)c--;// make an exception for the first one
                  temp=StringSubstr(text,prevs,c); c=0; prevs=i;

                  switch(glob)
                    {
                     case  0:  hdata[hsize-1].time   =StringToTime(temp);      break;
                     case  4:  hdata[hsize-1].price  =StringToDouble(temp);    break;
                     case  6:  hdata[hsize-1].balance=StringToDouble(temp);    break;
                     case  9:  hdata[hsize-1].entry  =StringToDouble(temp);    break;
                     default:  break;
                    }
                  glob++;
                 }
              }
            temp=StringSubstr(text,prevs,c);
            hdata[hsize-1].volume=StringToDouble(temp);
           }
         if(FileTell(han)>=FileSize(han))break;
        }
      FileClose(han);

      res=true;
     }
   else Print("File open failed "+folder_+".csv, error",GetLastError());
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSymBalance::InitData(string name_report_,string symbol_)
  {
   name_report=name_report_;
   symbol=symbol_;
   iPoint=1./SymbolInfoDouble(symbol,SYMBOL_POINT);
//--- reading history
   folder=name_report+" "+symbol;
   bool flag_process=ReadHistory(folder);
   if(flag_process)
     {
      if(IsStopped())return(false);
      beg=hdata[0].time;
      end=hdata[hsize-1].time;
      //--- get the amount of data
      int count=Bars(symbol,PERIOD_M1,beg,end);
      //---if no information is downloaded  
      int ff=0;
      while(!IsStopped() && count<=0)// download information
        {
         count=Bars(symbol,PERIOD_M1,beg,end);
         if(count>0)break;
         Sleep(10);
         // if the time of getting of information exceed the 1 minute, stop working
         ff++; if(ff>6000){Alert("No data for the symbol "+symbol); ExpertRemove(); Sleep(10); break;}
        }
      if(IsStopped())return(false);
      //--- move back by one bar  
      datetime current_time=TimeBars(0);
      int count_=Bars(symbol,PERIOD_M1,beg,current_time);
      if(count_>0){beg=TimeBars(count_+1);}
      //---if the information is downloaded 
      if(count>0)count_rates=CopyRates(symbol,PERIOD_M1,beg,end,rates);
      if(count_rates==-1)
        {Alert("No data for the symbol "+symbol+", or not enough bars in the window"); ExpertRemove();}
     }
   return(flag_process);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CSymBalance::Calculate_Balance()
  {
   int c=1;// counter of arrays of the indicator   
   Resize(count_rates);
   int C=0;
   array[0].time=0.;
   array[0].balance=0.;
   array[0].open=0.;
   array[0].bar1=0.;
   array[0].bar2=0.;
   array[0].close=0.;

   double temp=0,price_temp=0,k_entry=0; int k=1;
   double b0,b1,b2,b3;

   for(int i=1;i<count_rates;i++)
     {
      if(rates[i].time>=beg && rates[i].time<=end)// if we are in the history range
        {
         //--- --------------------------------------------------------------
         array[c].time=rates[i].time;
         bool fl=rates[i].open>rates[i].close;
         b0 = rates[i].open;
         b1 = fl?rates[i].high:rates[i].low;
         b2 = fl?rates[i].low:rates[i].high;
         b3 = rates[i].close;
         price_temp=rates[i-1].close;

         if(rates[i].time>hdata[C+1].time)// if there are several deals during 1 minute
           {if(i>0)i--; array[c].time+=k; k++;}// process everything by scrolling back time
         else k=1;// otherwise set the second shift to the initial state
         //--- if the time exceeds the current deal
         if(hdata[C+1].time>rates[i-1].time && hdata[C+1].time<=rates[i].time)
           {// go to the next deal
            if(C<hsize-1)C++; else break;
            if(hdata[C].entry==-1 && hdata[C].volume==0){temp=array[c].balance=hdata[C].balance;}
            else
              {
               array[c].balance=array[c-1].balance;
               temp=array[c-1].close;
              }
            array[c].open=temp;
            array[c].bar1=temp;
            array[c].bar2=temp;
            array[c].close=temp;
           }
         else// otherwise process the current deal
           {
            temp=array[c].balance=array[c-1].balance;
            //--- if is the last deal of position            
            if(hdata[C].entry==-1 && hdata[C].volume==0)
              {
               temp=array[c].balance;
               array[c].open=temp;
               array[c].bar1=temp;
               array[c].bar2=temp;
               array[c].close=temp;
              }
            else // otherwise, if it is not the last deal of position
              {
               k_entry=hdata[C].entry*2-1;
               double koeftemp=iPoint*hdata[C].volume*k_entry;
               temp=array[c-1].close;
               array[c].open=koeftemp*(price_temp-b0)+temp;
               array[c].bar1=koeftemp*(price_temp-b1)+temp;
               array[c].bar2=koeftemp*(price_temp-b2)+temp;
               array[c].close=koeftemp*(price_temp-b3)+temp;
              }
           }
         c++;// increase the counter
         if(c+1>=count_rates)Resize(c+1);
         //--- -------------------------------------------------------------- 
        }
     }
//--- assign information about the last deal
   temp=array[c].balance=hdata[hsize-1].balance;
   array[c].time=array[c-1].time+1;
   array[c].open=temp;
   array[c].bar1=temp;
   array[c].bar2=temp;
   array[c].close=temp;
   c++;
   Resize(c);
//Print("bsize=",c);
   return(c);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Balance_Process()                                                |
//+------------------------------------------------------------------+
void Balance_Process(string name_report,const string &trade_symbol[])
  {
   int total=ArraySize(trade_symbol);
//---
   CSymBalance *sym_bal[]; ArrayResize(sym_bal,total);
   for(int i=0;i<total;i++)
     {
      Comment("OnInit   Creating the balance history of "+trade_symbol[i]);
      sym_bal[i]=new CSymBalance();

      if(sym_bal[i].InitData(name_report,trade_symbol[i]))
        {
         sym_bal[i].Calculate_Balance();
         sym_bal[i].WriteFile(trade_symbol[i]+" sym");
        }
     }
//--- 

   CArrayLong *temp_time=new CArrayLong();

   int total_rates=sym_bal[0].bsize;
   for(int j=0;j<total_rates;j++)
      temp_time.Add(sym_bal[0].array[j].time);
   temp_time.Sort();

   for(int i=1;i<total;i++)
     {
      total_rates=sym_bal[i].bsize;
      for(int j=0;j<total_rates;j++)
        {
         if(temp_time.Search(sym_bal[i].array[j].time)==-1)
           {
            temp_time.Add(sym_bal[i].array[j].time);
            temp_time.Sort();
           }
        }
     }
   temp_time.Sort();

   total_rates=temp_time.Total();
   CBalance *all_bal=new CBalance(); ArrayResize(all_bal.array,total_rates);
   for(int i=0;i<total_rates;i++)all_bal.array[i].time=(datetime)temp_time.At(i);
   delete temp_time;
   for(int i=0;i<total_rates;i++)// initialize the structure
     {
      all_bal.array[i].balance =0.;
      all_bal.array[i].open    =0.;
      all_bal.array[i].bar1    =0.;
      all_bal.array[i].bar2    =0.;
      all_bal.array[i].close   =0.;
     }

   for(int i=0;i<total;i++)
     {
      int c=1,arr_end=sym_bal[i].bsize-1;
      for(int j=1;j<total_rates;j++)
        {
         if(j>arr_end)break;
         if(all_bal.array[j].time==sym_bal[i].array[c].time)
           {
            all_bal.array[j].balance += sym_bal[i].array[c].balance;
            all_bal.array[j].open    += sym_bal[i].array[c].open;
            all_bal.array[j].bar1    += sym_bal[i].array[c].bar1;
            all_bal.array[j].bar2    += sym_bal[i].array[c].bar2;
            all_bal.array[j].close   += sym_bal[i].array[c].close;
            if(c<arr_end)c++;
           }
         else
           {
            if(all_bal.array[j].time>sym_bal[i].array[c].time)
              {
               while(all_bal.array[j].time>sym_bal[i].array[c].time){if(c<arr_end)c++;else break;}
               all_bal.array[j].balance+=sym_bal[i].array[c].balance;
               all_bal.array[j].open    += sym_bal[i].array[c].open;
               all_bal.array[j].bar1    += sym_bal[i].array[c].bar1;
               all_bal.array[j].bar2    += sym_bal[i].array[c].bar2;
               all_bal.array[j].close   += sym_bal[i].array[c].close;
              }
            else
              {
               all_bal.array[j].balance += sym_bal[i].array[c-1].balance;
               all_bal.array[j].open    += sym_bal[i].array[c-1].open;
               all_bal.array[j].bar1    += sym_bal[i].array[c-1].bar1;
               all_bal.array[j].bar2    += sym_bal[i].array[c-1].bar2;
               all_bal.array[j].close+=sym_bal[i].array[c-1].close;
              }
           }
        }
     }
   all_bal.WriteFile("All sym");
   delete all_bal;
//--- 
   for(int i=0;i<total;i++) delete sym_bal[i];
   ArrayFree(sym_bal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
